#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#    Copyright © Manoel Vilela 2016
#
#    @project: Decorating
#     @author: Manoel Vilela
#      @email: manoel_vilela@engineer.com
#
"""
    An collection of usefull decorators for debug
    and time evaluation of functions flow
"""
from __future__ import unicode_literals

from functools import wraps
from time import time


def debug(function):
    """
    Function: debug
    Summary: decorator to debug a function
    Examples: at the execution of the function wrapped,
              the decorator will allows to print the
              input and output of each execution
    Attributes:
        @param (function): function
    Returns: wrapped function
    """

    @wraps(function)
    def _wrapper(*args, **kwargs):
        result = function(*args, **kwargs)
        for key, value in kwargs.items():
            args += tuple(['{}={!r}'.format(key, value)])
        if len(args) == 1:
            args = '({})'.format(args[0])
        print('@{0}{1} -> {2}'.format(function.__name__, args, result))
        _wrapper.last_output = [function.__name__, str(args), result]
        return result
    _wrapper.last_output = []
    return _wrapper


def counter(function):
    """
    Function: counter
    Summary: Decorator to count the number of a function is executed each time
    Examples: You can use that to had a progress of heally heavy
              computation without progress feedback
    Attributes:
        @param (function): function
    Returns: wrapped function
    """

    @wraps(function)
    def _wrapper(*args, **kwargs):
        _wrapper.count += 1
        res = function(*args, **kwargs)
        funcname = function.__name__
        count = _wrapper.count
        print("{} has been used: {}x".format(funcname, count))
        return res
    _wrapper.count = 0
    return _wrapper


def count_time(function):
    """
    Function: count_time
    Summary: get the time to finish a function
             print at the end that time to stdout
    Examples: <NONE>
    Attributes:
        @param (function): function
    Returns: wrapped function
    """
    @wraps(function)
    def _wrapper(*args, **kwargs):
        before = time()
        result = function(*args, **kwargs)
        diff = time() - before
        funcname = function.__name__
        print("{!r} func leave it {:.2f} ms to finish".format(funcname, diff))
        _wrapper.time = diff
        return result

    _wrapper.time = 0
    return _wrapper



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#    Copyright © Manoel Vilela 2016
#
#    @project: Decorating
#     @author: Manoel Vilela
#      @email: manoel_vilela@engineer.com
#
#
#                  ..        .;-:!!>>!!:--;..  .
#               .     .-:!>7?CO$QQQQ$OC?:--::-.....
#             .   .-!7?CO$QQQQQ$$$OCCCC?!:--:>>:;;;...
#           .  .->CCCC>?OO$$OO$$$$$$O$$$O?7>:>C$C7!:-;;.  .
#         .  .!?O$Q$$OOCC?CCO$QQQQQQ$$QQQ$$$OO$QNHH$C?>!:;  .
#        . .!OQQQHHHQQQQQ$$QQQQHQHQQQQQHQQHHHHHHHNNNHHQ$OC>.   .
#       . ;CQHHHHHHHHHHHHHHHHHHHHHHHQHHHHHHNNNNNNNNHNHNNHHH$!;
#        :OHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHNNNNNNNNNHHHHNNHNNO:-.
#       !OHNNNNNHHHHHHHHHHHHHHHHHHHHHHHHNNHHNNNNNNHHHHHHNHNNNH>;;.
#    . !OHNHHHHNHHHHHHHHHHHHHHHNNNNNNNHHHNNNNNNNNHNMNNNNHNNNHNN7  .
#     -OQNHNNNNNNHNHHNNHHNNNNNNNNNNNNNNHHHNNNNNHHNQ>!>OHNNHNNHHN$;
#    .?$HHNNNNNNNNNNHHNHHNNNNNNNNNNNNNNNNNNNNNNHNQ.;>:::CNNHNNHHNQ!.
#  . :$HNNNNNNHHNNHHNNNNHNNNNNNNNNNHHNHNNNNNNHHHN? !;  . -ONHNNHNO!-....
#   .?QHNNNNNNNNNNNNNNNNNNNNNNNNNHHNNHNHHNNNNNNNNC .;->!:..QNHHHN$!:-;.
# . :$HNNNHNNNNNNNNNNNNNNNNNNNNHHNM$QNHNNHNNNHQQNO  :$O>!- CNHHHHN$O7...
# . !QHNNNNNNNNNNNNNNNNNNNHHNNHNNO>;;?:7QNNNQQHQQH! .O7!; :HHHHHNNNH:...
# . !HHNNNNNNNNNNNNNNNNNNNHHHHH$>;.-!7C?O$HOHNNNNO?; -... CMHNNNHQH> ...
# . !HHNNNNNNNNNNNNNNNNNNNHHNN$>77OHNNHHQ7OC$QHHQ>;..  ..;$QOCOQOO!.....
# . !NNNNNNNNNNNNNNNNHHHHNHNNHC$CHO?Q$QQ? ?HQHQ$O$- .. .:??!-. :>- .....
#   .QNHNNNNNNNNNNNNNNNNNHHHN$!:-?; :7!.  ;>7NNNHQ7 .   .;;;..;:.......
#    ONHNNNNNNNNNNNNHO$HHMNNM?     .        .!HNHQO.      ;-.-:;....
#  . 7NHHNHNNNNNNNNHN?:!;>7:!!.      ..    .. 7NHQQ: .  ;;::>:;;.....
#    -!HNHHNNNNNNNNNNQ?>:;.   .             . .QNQQ? . ;>7C?:::.........
#      7NHNNHNNNNNHNQCCO$Q$O:.               . ?MHQ$.  >QO>!>7;.........
#    . .QHNNNNNHHHHHH$QHQNQ$>...             . >NHHN7 ;QO!>CC-..........
#       :-$HO$ONHHNHNNHO->CC; .;.            . 7NHHH$;?$>7O$:.......  ..
#         -$:--QHNQN>CMNQ:;--;;;....      .... ?NHNQ!OHCOOQ>;;;.....   .
#        . 7:  ;OH>?;.77-  ;;..:-      .....   ONHNQCHHHQHQ!--......  ..
#          .. . --.;;.       . ;O>:; .;........>$NNHNNNHHHH?-.... .   ..
#                ;..  ......;.;->>:::--;; .;..!>CNNQQNHNHNQ!....
#             . ;:. .......... .;:->?   -: .-.!HHHN7>NHHNH>;...      .
#              .!; ...   ........;!ONC>>C$7?O>7QNHNQ?HNNNO:...         .
#             :C!.;.   .... ...;-!?$NNNNNNNNNNNHHNHNNHHNQ7;...      ....
#            7$O-.. .        ...!$QNNHHHHHHHHHHHHNNHHNNQC-...     ......
#          .?Q$:......       ...-$NNNNHHHHNHNNNHHNNNNNQC>;..     .......


"""
    This is another LOL-zone

    LOOOOOOOOOOOOOL ART
"""
from __future__ import unicode_literals

# START
#     /\O    |    _O    |      O
#      /\/   |   //|_   |     /_
#     /\     |    |     |     |\
#    /  \    |   /|     |    / |
#  LOL  LOL  |   LLOL   |  LOLLOL
# -----------+----------+-----------


ARROWS = "←↖↑↗→↘↓↙"
BALLS = ".oO@*"
BRAILY = "⣾⣽⣻⢿⡿⣟⣯"
CIRCLE1 = "◐◓◑◒"
CIRCLE2 = "◴◷◶◵"
CYCLE = "|/-\\"
EYES = ["◡◡" "⊙⊙" "◠◠"]
HPULSE = "▉▊▋▌▍▎▏▎▍▌▋▊▉"
PIPES = "┤┘┴└├┌┬┐"
ROTATING = "▖▘▝▗"
SQUARES = "◰◳◲◱"
TRIANGULES = "◢◣◤◥"
VPULSE = '▁▂▃▄▅▆▇▆▅▄▃▁'
WAVE = "⠁⠂⠄⡀⢀⠠⠐⠈"


# END
#     /\O    |    _O    |      O
#      /\/   |   //|_   |     /_
#     /\     |    |     |     |\
#    /  \    |   /|     |    / |
#  LOL  LOL  |   LLOL   |  LOLLOL
# -----------+----------+-----------



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#    Copyright © Manoel Vilela 2016
#
#    @project: Decorating
#     @author: Manoel Vilela
#      @email: manoel_vilela@engineer.com
#

"""
    Module focused in termcolor operations

    If the exection is not attatched in any tty,
    so colored is disabled
"""
from __future__ import unicode_literals

import sys

COLORED = True
if not sys.stdout.isatty() or sys.platform == 'win32':
    COLORED = False  # pragma: no cover

COLOR_MAP = {
    'brown': '\033[{style};30m',
    'red': '\033[{style};31m',
    'green': '\033[{style};32m',
    'yellow': '\033[{style};33m',
    'blue': '\033[{style};34m',
    'pink': '\033[{style};35m',
    'cyan': '\033[{style};36m',
    'gray': '\033[{style};37m',
    'white': '\033[{style};40m',
    'reset': '\033[00;00m'
}

STYLE_MAP = {
    'normal': '00',
    'bold': '01',
    'underline': '04',
}


def colorize(printable, color, style='normal', autoreset=True):
    """Colorize some message with ANSI colors specification

    :param printable: interface whose has __str__ or __repr__ method
    :param color: the colors defined in COLOR_MAP to colorize the text
    :style: can be 'normal', 'bold' or 'underline'

    :returns: the 'printable' colorized with style
    """
    if not COLORED:  # disable color
        return printable
    if color not in COLOR_MAP:
        raise RuntimeError('invalid color set, no {}'.format(color))

    return '{color}{printable}{reset}'.format(
        printable=printable,
        color=COLOR_MAP[color].format(style=STYLE_MAP[style]),
        reset=COLOR_MAP['reset'] if autoreset else ''
    )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#    Copyright © Manoel Vilela 2016
#
#    @project: Decorating
#     @author: Manoel Vilela
#      @email: manoel_vilela@engineer.com
#

"""
    DECORATING: A MODULE OF DECORATORS FROM HELL

    You have that collection of decorators:

    * animated: create animations on terminal until the result's returns
    * cache: returns without reprocess if the give input was already processed
    * counter: count the number of times whose the decorated function is called
    * debug: when returns, print this pattern: @function(args) -> result
    * count_time: count the time of the function decorated did need to return
"""

from decorating.animation import animated, writing
from decorating.debugging import count_time, counter, debug
from decorating.general import cache

__version__ = '0.6'
__author__ = 'Manoel Vilela'
__email__ = 'manoel_vilela@engineer.com'
__url__ = 'https://github.com/ryukinix/decorating'


__all__ = [
    'animated',
    'cache',
    'counter',
    'debug',
    'count_time',
    'writing'
]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#    Copyright © Manoel Vilela 2016
#
#    @project: Decorating
#     @author: Manoel Vilela
#      @email: manoel_vilela@engineer.com
#
# pylint: disable=too-few-public-methods

"""

    This module have a collection of Streams class
    used to implement:

    * Unbuffered(Stream) :: stream wrapper auto flushured
    * Animation(Unbuferred) :: stream with erase methods
    * Clean(Unbuffered) ::  stream with handling paralell conflicts
    * Writing(Unbuffered) :: stream for writing delayed typing

"""
from __future__ import unicode_literals

import time
import re
from threading import Lock
from decorating.base import Stream


class Unbuffered(Stream):

    """Unbuffered whose flush automaticly

    That way we don't need flush after a write.

    """

    lock = Lock()

    def __init__(self, stream):
        super(Unbuffered, self).__init__(stream)
        self.stream = stream

    def write(self, message, flush=True):
        """
        Function: write
        Summary: write method on the default stream
        Examples: >>> stream.write('message')
                  'message'
        Attributes:
            @param (message): str-like content to send on stream
            @param (flush) default=True: flush the stdout after write
        Returns: None
        """
        self.stream.write(message)
        if flush:
            self.stream.flush()

    def __getattr__(self, attr):
        return getattr(self.stream, attr)


class Animation(Unbuffered):

    """A stream unbuffered whose write & erase at interval

    After you write something, you can easily clean the buffer
    and restart the points of the older message.
    stream = Animation(stream, delay=0.5)
    self.write('message')

    """

    last_message = ''
    ansi_escape = re.compile(r'\x1b[^m]*m')

    def __init__(self, stream, interval=0.05):
        super(Animation, self).__init__(stream)
        self.interval = interval

    def write(self, message, autoerase=True):
        """Send something for stdout and erased after delay"""
        super(Animation, self).write(message)
        self.last_message = message
        if autoerase:
            time.sleep(self.interval)
            self.erase(message)

    def erase(self, message=None):
        """Erase something whose you write before: message"""
        if not message:
            message = self.last_message
        # Move cursor to the beginning of line
        super(Animation, self).write("\033[G")
        # Erase in line from cursor
        super(Animation, self).write("\033[K")

    def __getattr__(self, attr):
        return getattr(self.stream, attr)


class Clean(Unbuffered):
    """A stream wrapper to prepend '\n' in each write

    This is used to not break the animations when he is activated

    So in the start_animation we do:
        sys.stdout = Clean(sys.stdout, <paralell-stream>)

    In the stop_animation we do:
        sys.stdout = sys.__stdout__Whose paralell_stream is a Animation object.

    """

    def __init__(self, stream, paralell_stream):
        super(Clean, self).__init__(stream)
        self.paralell_stream = paralell_stream

    def write(self, message, flush=False):
        """Write something on the default stream with a prefixed message"""
        # this need be threadsafe because the concurrent spinning running on
        # the stderr
        with self.lock:
            self.paralell_stream.erase()
            super(Clean, self).write(message, flush)
            # for some reason I need to put a '\n' here to correct
            # print of the message, if don't put this, the print internal
            # during the animation is not printed.
            # however, this create a other problem: excess of newlines


class Writting(Unbuffered):

    """
        The Writting stream is a delayed stream whose
        simulate an user Writting something.

  The base class is the AnimationStream


    """

    def __init__(self, stream, delay=0.08):
        super(Writting, self).__init__(stream)
        self.delay = delay

    def write(self, message, flush=True):
        if isinstance(message, bytes):  # pragma: no cover
            message = message.decode('utf-8')

        """A Writting like write method, delayed at each char"""
        for char in message:
            time.sleep(self.delay * (4 if char == '\n' else 1))
            super(Writting, self).write(char, flush)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#    Copyright © Manoel Vilela 2016
#
#    @project: Decorating
#     @author: Manoel Vilela
#      @email: manoel_vilela@engineer.com
#
# pylint: disable=redefined-builtin
# pylint: disable=invalid-name

"""
    An collection of usefull decorators for debug
    and time evaluation of functions flow
"""

# stdlib
from functools import wraps
import sys

PY2 = sys.version_info[0] == 2
PY3 = sys.version_info[0] == 3


if PY2:  # pragma: no cover
    from itertools import izip
    zip = izip
else:  # pragma: no cover
    zip = zip


def with_metaclass(meta, *bases):
    """Create a base class with a metaclass."""
    # This requires a bit of explanation: the basic idea is to make a dummy
    # metaclass for one level of class instantiation that replaces itself with
    # the actual metaclass.

    # Copied from `six' library.
    # Copyright (c) 2010-2015 Benjamin Peterson
    # License: MIT

    class metaclass(meta):
        """Dummy metaclass"""
        def __new__(cls, name, this_bases, d):
            return meta(name, bases, d)
    return type.__new__(metaclass, 'temporary_class', (), {})


def cache(function):
    """
    Function: cache
    Summary: Decorator used to cache the input->output
    Examples: An fib memoized executes at O(1) time
              instead O(e^n)
    Attributes:
        @param (function): function
    Returns: wrapped function

    TODO: Give support to functions with kwargs
    """

    memory = {}
    miss = object()

    @wraps(function)
    def _wrapper(*args):
        result = memory.get(args, miss)
        if result is miss:
            _wrapper.call += 1
            result = function(*args)
            memory[args] = result
        return result
    _wrapper.call = 0
    return _wrapper



from boto import ec2
from operator import itemgetter

def get_all_instances(region='us-east-1'):
    instances = []
    index = 1
    conn = ec2.connect_to_region(region)
    reservations = conn.get_all_instances()
    for res in reservations:
        for inst in res.instances:
            id = inst.id
            name = inst.tags.get('Name', 'NoName')
            location = inst.placement
            launch_time = str(inst.launch_time)
            size = inst.instance_type
            public_ip = inst.ip_address or ''
            private_ip = inst.private_ip_address or ''
            pub_dns = inst.public_dns_name or ''
            priv_dns = inst.private_dns_name or ''
            vpc_id = inst.vpc_id or ''
            subnet_id = inst.subnet_id or ''
            env = inst.tags.get('Env', '')
            role = inst.tags.get('Role', '')
            master = inst.tags.get('master', '')
            cfn_logical_id = inst.tags.get('aws:cloudformation:logical-id', '')
            cfn_stack_id = inst.tags.get('aws:cloudformation:stack-id', '')
            cfn_stack_name = inst.tags.get('aws:cloudformation:stack-name', '')
            as_group_name = inst.tags.get('aws:autoscaling:groupName', '')
            # Stringing tags that are not relevant to indexing
            other_tags = {}
            for (key, val) in inst.tags.items():
                if 'aws' not in key and 'Name' not in key and 'Hostname' not in key:
                    other_tags[key] = str(val)
            tags_txt = ', '.join("{!s}={!r}".format(key, val) for (key, val) in sorted(other_tags.items()))
            monitored = inst.monitored
            instances.append({'id': id, 'name': name, 'location': location,
                                'size': size,
                                'pub_ip': public_ip, 'priv_ip': private_ip, 'pub_dns': pub_dns,
                                'priv_dns': priv_dns,
                                'status': inst.state, 'vpc_id': vpc_id, 'subnet_id': subnet_id,
                                'monitored': monitored, 'tags': dict(inst.tags), 'tags_txt': tags_txt,
                                'env': env, 'role': role, 'master': master,
                                'cfn_logical_id': cfn_logical_id, 'cfn_stack_id': cfn_stack_id,
                                'cfn_stack_name': cfn_stack_name,
                                'as_group_name': as_group_name,
                                'launch_time': launch_time})
    sorted_instances = sorted(instances, key=itemgetter('env', 'role', 'launch_time'))
    for s in sorted_instances:
        s['index'] = index
        index += 1
    return sorted_instances


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#    Copyright © Manoel Vilela 2016
#
#    @project: Decorating
#     @author: Manoel Vilela
#      @email: manoel_vilela@engineer.com
#
# pylint: disable=no-member
# pylint: disable=too-few-public-methods


"""
    The base class for creating new Decorators

    * Decorator: A base class for creating new decorators

"""
from __future__ import unicode_literals

from functools import wraps
from warnings import warn
from .base import DecoratorManager


# A UNINTENDED LOL-ZONE: SORRY FOR THIS
# -----------+----------+-----------
#     /\O    |    _O    |      O
#      /\/   |   //|_   |     /_
#     /\     |    |     |     |\
#    /  \    |   /|     |    / |
#  LOL  LOL  |   LLOL   |  LOLLOL
# -----------+----------+-----------
# FULL FEATURED BLACK MAGICK ENABLED


class Decorator(DecoratorManager):

    """Decorator base class to keep easy creating more decorators

    triggers:
        self.start
        self.stop

    context_manager:
        self.__enter__
        self.__exit__

    Only this is in generall necessary to implement the class you are writing,
    like this:

    class Wired(Decorator):
        def __init__(self, user='Lain')
            self.user = user
        def start(self):
            self.login()

        def stop(self):
            self.logoff()

        def login(self):
            print('Welcome to the Wired, {user}!'.format(user=self.user))

        def logoff(self):
            print('Close this world, open the next!'.)



    And all the black magic is done for you behind the scenes. In theory,
    you can  use the decorator in these way:

    @Wired('lain')
    def foo():
        pass

    @Wired(argument='banana')
    def bar():
        pass

    @Wired
    def lain():
        pass

    @Wired()
    def death():
        pass

    And all are okay! As well, natively, you have support to use as
    context managers.

    So that you can handle that way:

    with Wired:
        print("Download the Knight files...")

    with Wired():
        print("Underlying bugs not anymore")

    with Wired("Lerax"):
        print("I'm exists?")

    with Wired(user="Lerax"):
        print("I don't have the real answer.")

    And all occurs be fine like you thinks this do.


    """

    # a map of instances to handle between the various forms
    # of using decorators, like @foo() or @foo.
    instances = []

    @classmethod
    def __call__(cls, *args, **kwargs):
        instance = cls.recreate(*args, **kwargs)
        cls.instances.append(instance)
        if any(args) and callable(args[0]):  # pass a function/class
            return instance._over_wrapper(args[0])

        return instance

    def _over_wrapper(self, function):
        @wraps(function)
        def _wrapper(*args, **kargs):
            self.start()
            result = function(*args, **kargs)
            self.stop()
            return result
        return _wrapper

    @classmethod
    def default_arguments(cls):
        """Returns the available kwargs of the called class"""
        func = cls.__init__
        args = func.__code__.co_varnames
        defaults = func.__defaults__
        index = -len(defaults)
        return {k: v for k, v in zip(args[index:], defaults)}

    @classmethod
    def recreate(cls, *args, **kwargs):
        """Recreate the class based in your args, multiple uses"""
        cls.check_arguments(kwargs)
        first_is_callable = True if any(args) and callable(args[0]) else False
        signature = cls.default_arguments()
        allowed_arguments = {k: v for k, v in kwargs.items() if k in signature}
        if (any(allowed_arguments) or any(args)) and not first_is_callable:
            if any(args) and not first_is_callable:
                return cls(args[0], **allowed_arguments)
            elif any(allowed_arguments):
                return cls(**allowed_arguments)

        return cls.instances[-1] if any(cls.instances) else cls()

    @classmethod
    def check_arguments(cls, passed):
        """Put warnings of arguments whose can't be handle by the class"""
        defaults = list(cls.default_arguments().keys())
        template = ("Pass arg {argument:!r} in {cname:!r}, can be a typo? "
                    "Supported key arguments: {defaults}")
        fails = []
        for arg in passed:
            if arg not in defaults:
                warn(template.format(argument=arg,
                                     cname=cls.__name__,
                                     defaults=defaults))
                fails.append(arg)

        return any(fails)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#    Copyright © Manoel Vilela 2016
#
#    @project: Decorating
#     @author: Manoel Vilela
#      @email: manoel_vilela@engineer.com
#
# pylint: disable=no-member
# pylint: disable=too-few-public-methods


"""
    Abstract Classes to do composition by inheterince
    and some other utitities from base clases

    * Stream: Abstract Class for implementation of a Stream

    * Decorator: Abstract Class for creating new decorators

"""
from __future__ import unicode_literals
from abc import abstractmethod, ABCMeta
from .general import with_metaclass


class Stream(with_metaclass(ABCMeta)):
    """A base class whose is specify a Stream is

    We need at least a stream on init and a
    message param on write method

    """

    @abstractmethod
    def __init__(self, stream, **kargs):
        pass

    @abstractmethod
    def write(self, message, optional=None):
        """a write method interfacing sys.stdout or sys.stderr"""
        pass


class DecoratorManager(with_metaclass(ABCMeta)):
    """Decorator-Context-Manager base class to keep easy creating more decorators

    argument: can be empty or a callable object (function or class)
    """

    @abstractmethod
    def __call__(self, function):
        """Base class to handle all the implementation of decorators"""
        pass

    @abstractmethod
    def start(self):
        """You active here your pre-fucking crazy feature"""
        pass

    @abstractmethod
    def stop(self):
        """You can deactivate any behavior re-writing your method here"""
        pass

    def __enter__(self):
        """Activated when enter in a context-manager (with keyword)"""
        self.start()

    def __exit__(self, *args):
        """Triggered when exit from a block of context-manager"""
        self.stop()



#!/usr/bin/env python

import argparse
import logging
import httplib

from settings import *
from version import __version__


def main():
    """ Main Loop """
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '-v', '--verbose', action='store_true', default=False,
        help='Increase log verbosity')
    parser.add_argument(
        '-d', '--debug', action='store_true', default=False,
        help='Debug level verbosity')
    parser.add_argument(
        '--version', action='store_true', default=False,
        help='Show version information', dest='version')

    args = parser.parse_args()
    if args.verbose:
        httplib.HTTPConnection.debuglevel = 1
        logging.basicConfig(level=logging.INFO)
        logging.getLogger().setLevel(logging.INFO)
        requests_log = logging.getLogger("requests.packages.urllib3")
        requests_log.setLevel(logging.INFO)
        requests_log.propagate = True
    if args.debug:
        httplib.HTTPConnection.debuglevel = 1
        logging.basicConfig(level=logging.DEBUG)
        logging.getLogger().setLevel(logging.DEBUG)
        requests_log = logging.getLogger("requests.packages.urllib3")
        requests_log.setLevel(logging.DEBUG)
        requests_log.propagate = True
    if args.version:
        show_version()


def show_version():
    print "Sample version %s " % __version__


if __name__ == '__main__':
    main()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#    Copyright © Manoel Vilela 2016
#
#    @project: Decorating
#     @author: Manoel Vilela
#      @email: manoel_vilela@engineer.com
#
# pylint: disable=no-member
# pylint: disable=C0103
# pylint: disable=too-few-public-methods

"""

    This module was be done to handle the beautiful animation using
    the sin function (whose cause a pulse in the stdout).

    Some examples of using is here:

        @animated
        def slow():
            heavy_stuff()

        As well with custom messages
        @animated('WOOOOW')
        def download_the_universe():
            while True:
                pass

        with animated('loool'):
            stuff_from_hell()

        @writing
        def printer():
            lot_of_messages()

        with writing(delay=0.5):
            print("L  O  L => IS NOT THE STUPID GAME LOL, LOL.")


"""
from __future__ import unicode_literals
import signal
import sys
import threading
from math import sin
from functools import partial
from itertools import cycle
from . import decorator, color, stream, asciiart
from .general import zip


# THIS IS A LOL ZONE
#    /\O    |    _O    |      O
#     /\/   |   //|_   |     /_
#    /\     |    |     |     |\
#   /  \    |   /|     |    / |
# LOL  LOL  |   LLOL   |  LOLLOL


class SpinnerController(object):
    """Variables to controlling the spinner in parallel

    Bias: the initial value of the padding function
          is used here because after a animation stop
          and other is started in sequence, the padding
          for a semantic view need be in the same place.

    running: variable signal-like to stop the thread
          on the main loop doing the animation

    message: the actual messaging on the spinner

    stream: the stream to do the animation, needs
            implement the AbstractClass stream.Stream

    """

    bias = 0
    running = False
    message = ''
    stream = stream.Animation(sys.stderr)
    fpadding = None


class AnimationController(object):
    """Used to controlling thread & running

    context: the context level added +1 at each nested 'with'
    running: the object running in the actual moment

    """
    context = 0
    thread = None
    messages = []


def space_wave(phase, char=asciiart.WAVE, amplitude=12, frequency=0.1):
    """
    Function: space_wave
    Summary: This function is used to generate a wave-like padding
             spacement based on the variable lambda
    Examples: >>> print('\n'.join(space_wave(x) for x in range(100))
              █
              ███
              ████
              ██████
              ███████
              ████████
              █████████
              ██████████
              ██████████
              ██████████
              ██████████
              ██████████
              ██████████
              █████████
              ████████
              ███████
              █████
              ████
              ██
              █

    Attributes:
        @param (phase): your positive variable, can be a int or float
        @param (char) default='█': the char to construct the space_wave
        @param (amplitude) default=10: a float/int number to describe
                                       how long is the space_wave max
        @param (frequency) default=0.1: the speed of change
    Returns: a unique string of a sequence of 'char'
    """
    wave = cycle(char)
    return ''.join((next(wave) for x in range
                    (int((amplitude + 1) * abs(sin(frequency * (phase)))))))


def _spinner(control):
    if not sys.stdout.isatty():  # not send to pipe/redirection
        return

    colorize_no_reset = partial(color.colorize, autoreset=False)

    template = '{padding} {start} {message} {end}'
    NBRAILY = ''.join(x * 5 for x in asciiart.BRAILY)
    iterator = zip(cycle(NBRAILY), cycle(asciiart.VPULSE))
    for i, (start, end) in enumerate(iterator):
        padding = control.fpadding(i + control.bias)
        message = '\r' + template.format(
            message=colorize_no_reset(control.message, 'red'),
            padding=colorize_no_reset(padding, 'blue'),
            start=colorize_no_reset(start, 'cyan'),
            end=color.colorize(end, 'cyan')
        )
        with control.stream.lock:
            control.stream.write(message)
        if not control.running:
            control.bias = i
            break
    control.stream.erase(message)

# D
#   E
#     C
#       O
#         R
#           A
#             T
#               O
#                 R
#                   S


# deal with it
class AnimatedDecorator(decorator.Decorator):

    """The animated decorator from hell

    You can use this these way:

        @animated
        def slow():
            heavy_stuff()

        As well with custom messages
        @animated('WOOOOW')
        def download_the_universe():
            while True:
                pass

        with animated('loool'):
            stuff_from_hell()
    """

    # if nothing is passed, so this is the message
    default_message = 'loading'
    # to handle various decorated functions
    spinner = SpinnerController()
    # to know if some instance of this class is running
    # and proper handle that, like ctrl + c and exits
    animation = AnimationController()

    def __init__(self, message=None, fpadding=space_wave):
        super(AnimatedDecorator, self).__init__()
        self.message = message
        self.spinner.fpadding = fpadding

    def start(self, autopush=True):
        """Start a new animation instance"""
        if autopush:
            self.push_message(self.message)
        self.spinner.message = ' - '.join(self.animation.messages)
        if not self.spinner.running:
            self.animation.thread = threading.Thread(target=_spinner,
                                                     args=(self.spinner,))
            self.spinner.running = True
            self.animation.thread.start()
            sys.stdout = stream.Clean(sys.stdout, self.spinner.stream)

    @classmethod
    def stop(cls):
        """Stop the thread animation gracefully and reset_message"""
        if cls.spinner.running:
            cls.spinner.running = False
            cls.animation.thread.join()

        if any(cls.animation.messages):
            cls.pop_message()

        sys.stdout = sys.__stdout__

    def __enter__(self):
        self.animation.context += 1
        self.start()

    def __exit__(self, *args):
        # if the context manager doesn't running yet
        self.animation.context -= 1
        self.pop_message()
        if self.animation.context == 0:
            self.stop()
        else:
            self.start(autopush=False)

    @classmethod
    def push_message(cls, message):
        """Push a new message for the public messages stack"""
        return cls.animation.messages.append(message)

    @classmethod
    def pop_message(cls):
        """Pop a new message (last) from the public message stack"""
        return cls.animation.messages.pop(-1)

    @classmethod
    def __call__(cls, *args, **kwargs):
        obj = super(AnimatedDecorator, cls).__call__(*args, **kwargs)
        if any(cls.instances):
            last_instance = cls.instances[-1]
            last_instance.message = last_instance.auto_message(args)
        elif isinstance(obj, cls):
            obj.message = obj.auto_message(args)
        return obj

    def auto_message(self, args):
        """Try guess the message by the args passed

        args: a set of args passed on the wrapper __call__ in
              the definition above.

        if the object already have some message (defined in __init__),
        we don't change that. If the first arg is a function, so is decorated
        without argument, use the func name as the message.

        If not self.message anyway, use the default_message global,
        another else  use the default self.message already

        """
        if any(args) and callable(args[0]) and not self.message:
            return args[0].__name__
        elif not self.message:
            return self.default_message
        else:
            return self.message


class WritingDecorator(decorator.Decorator):

    """A writing class context to simulate a delayed stream

    You can do something like that:

    with writing(delay=0.3):
        print('LOL!!! This is so awesome!')

    Or, as expected for this lib, using as decorator!

    @writing
    def somebody_talking():
        print("Oh man... I'm so sad. Why I even exists?")
        print("I don't meeting anybody")
        print("I don't want answer my phone")
        print("I don't even to live")
        print("But dying is so hassle.")
        print("I'd wish just disappears.")

    delay: the down speed of writing, more bigger, more slow.


    """

    # to handle nested streams
    streams = []
    enabled = True

    def __init__(self, delay=0.05):
        super(WritingDecorator, self).__init__()
        self.stream = stream.Writting(sys.stdout, delay=delay)
        if not self.enabled:
            self.stream = sys.__stdout__

    def start(self):
        """Activate the TypingStream on stdout"""
        self.streams.append(sys.stdout)
        sys.stdout = self.stream

    @classmethod
    def stop(cls):
        """Change back the normal stdout after the end"""
        if any(cls.streams):
            sys.stdout = cls.streams.pop(-1)
        else:
            sys.stdout = sys.__stdout__


def _killed():
    AnimatedDecorator.stop()
    WritingDecorator.stop()
    AnimatedDecorator.spinner.stream.dump.close()
    raise KeyboardInterrupt

signal.signal(signal.SIGINT, lambda x, y: _killed())

animated = AnimatedDecorator('loading')
writing = WritingDecorator()


__all__ = [
    'animated',
    'writing'
]
